package sun.font;

import java.awt.geom.Point2D;
import java.util.concurrent.ConcurrentHashMap;
import java.util.WeakHashMap;
import java.lang.ref.SoftReference;

public final class SunLayoutEngine implements GlyphLayout.LayoutEngine, GlyphLayout.LayoutEngineFactory
{
    private GlyphLayout.LayoutEngineKey key;
    private static GlyphLayout.LayoutEngineFactory instance;
    private SoftReference cacheref;
    static WeakHashMap<Font2D, Boolean> aatInfo;
    
    private static native void initGVIDs();
    
    public static GlyphLayout.LayoutEngineFactory instance() {
        if (SunLayoutEngine.instance == null) {
            SunLayoutEngine.instance = new SunLayoutEngine();
        }
        return SunLayoutEngine.instance;
    }
    
    private SunLayoutEngine() {
        this.cacheref = new SoftReference(null);
    }
    
    @Override
    public GlyphLayout.LayoutEngine getEngine(Font2D font2D, int n, int n2) {
        return this.getEngine(new GlyphLayout.LayoutEngineKey(font2D, n, n2));
    }
    
    @Override
    public GlyphLayout.LayoutEngine getEngine(GlyphLayout.LayoutEngineKey key) {
        ConcurrentHashMap<GlyphLayout.LayoutEngineKey, GlyphLayout.LayoutEngine> referent = (ConcurrentHashMap)this.cacheref.get();
        if (referent == null) {
            referent = new ConcurrentHashMap();
            this.cacheref = new SoftReference(referent);
        }
        GlyphLayout.LayoutEngine value = (Object)referent.get(key);
        if (value == null) {
            GlyphLayout.LayoutEngineKey copy = key.copy();
            value = new SunLayoutEngine(copy);
            referent.put(copy, value);
            return value;
        }
        return value;
    }
    
    private SunLayoutEngine(GlyphLayout.LayoutEngineKey key) {
        this.cacheref = new SoftReference(null);
        this.key = key;
    }
    
    private boolean isAAT(Font2D font2D) {
        Boolean b;
        synchronized (SunLayoutEngine.aatInfo) {
            b = (Boolean)SunLayoutEngine.aatInfo.get(font2D);
        }
        if (b != null) {
            return (boolean)b;
        }
        boolean b2 = false;
        if ((font2D instanceof TrueTypeFont)) {
            TrueTypeFont trueTypeFont = (TrueTypeFont)font2D;
            b2 = (trueTypeFont.getDirectoryEntry(1836020344) != null || trueTypeFont.getDirectoryEntry(1836020340) != null);
        }
        else if ((font2D instanceof PhysicalFont)) {
            PhysicalFont physicalFont = (PhysicalFont)font2D;
            b2 = (physicalFont.getTableBytes(1836020344) != null || physicalFont.getTableBytes(1836020340) != null);
        }
        synchronized (SunLayoutEngine.aatInfo) {
            SunLayoutEngine.aatInfo.put(font2D, Boolean.valueOf((boolean)b2));
        }
        return b2;
    }
    
    @Override
    public void layout(FontStrikeDesc fontStrikeDesc, float[] array, int n, int n2, TextRecord textRecord, int n3, Point2D.Float float1, GlyphLayout.GVData gvData) {
        Font2D font = this.key.font();
        nativeLayout(font, font.getStrike(fontStrikeDesc), array, n, n2, textRecord.text, textRecord.start, textRecord.limit, textRecord.min, textRecord.max, this.key.script(), this.key.lang(), n3, float1, gvData, font.getUnitsPerEm(), ((n3 & Integer.MIN_VALUE) != 0x0 && this.isAAT(font)) ? 0L : font.getLayoutTableCache());
    }
    
    private static native void nativeLayout(Font2D p0, FontStrike p1, float[] p2, int p3, int p4, char[] p5, int p6, int p7, int p8, int p9, int p10, int p11, int p12, Point2D.Float p13, GlyphLayout.GVData p14, long p15, long p16);
    
    static {
        FontManagerNativeLibrary.load();
        initGVIDs();
        SunLayoutEngine.aatInfo = new WeakHashMap();
    }
}
